#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# imports.
from dev0s.classes.defaults.objects import *
from dev0s.classes.defaults.defaults import defaults
from dev0s.classes.defaults.exceptions import Exceptions
from dev0s.classes.response import response as _response_
from dev0s.classes.requests import requests as _requests_
from dev0s.classes import code
import requests as __requests__

# the database class.
class Database(Traceback):
	def __init__(self, 
		# the path to the directory (str) (#1)
		path=None, 
		# root permission required.
		sudo=False,
	):
		
		# docs.
		DOCS = {
			"module":"dev0s.database.Database", 
			"initialized":False,
			"description":[], 
			"chapter": "Database", }

		# traceback.
		Traceback.__init__(self, traceback="Database")

		# checks.
		if path == None: raise Exceptions.InvalidUsage(self.__traceback__()+" Define parameter [path].")

		# args.
		self.path = gfp.clean(path)
		self.sudo = sudo

		# sys args.
		self.__cache__ = {}

		# attributes.
		self.dir = self.directory = Directory(self.path)

		# checks.
		if not self.dir.fp.exists(sudo=sudo):
			if self.sudo: _response_.log(f"&ORANGE&Root permission&END& required to create database [{self.path}].")
			Files.create(str(self.path), sudo=self.sudo, directory=True, permission=700, owner=defaults.vars.user, group=defaults.vars.group,)

		# copy objects.
		self.fp = self.file_path = self.dir.fp
		self.ownership = self.fp.ownership
		self.permission = self.fp.permission

		# copy functions.
		self.join = self.dir.join
		self.subpath = self.dir.subpath
		self.fullpath = self.dir.fullpath

		#

	# load an object.
	def load(self, 
		# the sub path (str, FilePath) (#1).
		path=None, 
		# the data format [bool, str, int, float, dict, list] (str) (#2).
		format="dict", 
		# the default data (bool, str, int, float, dict, list).
		default=None,
	):
		if path == None: return _response_.error(self.__traceback__(function="load")+" Define parameter: [path].")
		path = str(path)
		subpath = path
		path = gfp.clean(f"{self.path}/{path}")
		format = self.__serialize_format__(format)
		if default != None and not Files.exists(path):
			response = self.save(path=subpath, data=default, format=format)
			if not response.success: return response
		try:
			if format in ["bytes"]:
				obj = Bytes(path=path, load=True)
			elif format in ["bool"]:
				obj = Boolean(path=path, load=True)
			elif format in ["str"]:
				obj = String(path=path, load=True)
			elif format in ["int", "float"]:
				obj = Integer(path=path, load=True)
			elif format in ["list"]:
				obj = Array(path=path, load=True)
			elif format in ["dict"]:
				obj = Dictionary(path=path, load=True)
		except Exception as e: return _response_.error(str(e))
		return _response_.success(f"Successfully loaded [{path}].", {
			"data":obj,
		})

	# save an object.
	def save(self, 
		# the sub path (str, FilePath) (#1).
		path=None, 
		# the data to save (bool, str, int, float, dict, list) (#2)
		data=None, 
		# the data format [bool, str, int, float, dict, list] (str) (#3).
		format="dict",
		# with overwrite disabled the dict format data is inserted into the existing data (bool).
		overwrite=False, 
	):
		if path == None: return _response_.error(self.__traceback__(function="save")+" Define parameter: [path].")
		if data == None: return _response_.error(self.__traceback__(function="save")+" Define parameter: [data].")
		path = str(path)
		path = gfp.clean(f"{self.path}/{path}")
		format = self.__serialize_format__(format)
		try:
			if not Files.exists(path=gfp.base(path)): Files.create(path=gfp.base(path), directory=True)
		except ValueError: a=1
		try:
			if format in ["bytes"]:
				obj = Bytes(data, path=path)
			elif format in ["bool"]:
				obj = Boolean(data, path=path)
			elif format in ["str"]:
				obj = String(data, path=path)
			elif format in ["int", "float"]:
				obj = Integer(data, path=path)
			elif format in ["list"]:
				obj = Array(data, path=path)
			elif format in ["dict"]:
				obj = Dictionary({}, path=path)
			if format in ["dict"]:
				if overwrite:
					obj.save(dictionary=data)
				else:
					if obj.fp.exists():
						obj.load()
					else:
						obj.dictionary = {}
					obj.insert(data)
					obj.save()
			else:
				obj.save()
		except Exception as e: return _response_.error(str(e))
		return _response_.success(f"Successfully saved [{path}].")

	# delete an object.
	def delete(self, 
		# the sub path (str, FilePath) (#1).
		path=None,
	):
		if path == None: return _response_.error(self.__traceback__(function="delete")+" Define parameter: [path].")
		path = gfp.clean(f"{self.path}/{path}")
		try:
			Files.delete(path=path)
		except Exception as e: return _response_.error(str(e))
		if Files.exists(path): return _response_.error(f"Failed to delete [{path}].")
		return _response_.success(f"Successfully deleted [{path}].")
	
	# get the paths of a directory.
	def paths(self,
		# the sub path (leave None to use the root path) (str FilePath)
		path=None,
		# get recursively (bool).
		recursive=False, 
		# get files only (bool).
		files_only=False,
		# get firs only (bool). 
		dirs_only=False, 
		# also get empty dirs (bool).
		empty_dirs=True, 
		# get the full path (bool).
		full_path=False,
		# the banned full paths (list).
		banned=[], 
		# the banned names (list).
		banned_names=[".DS_Store"], 
		# the banend base names (list).
		banned_basenames=["__pycache__"], 
		# the allowed extensions (list).
		extensions=["*"],
	):

		# checks.
		#if path == None: return _response_.error(self.__traceback__(function="names")+" Define parameter: [path].")
		if path == None: path = self.path
		else: path = self.join(path)
		path = str(path)
		if not Files.directory(path):
			raise dev0s.exceptions.InvalidUsage(f"Defined path [{path}] is not a directory.")

		# handler.
		paths = Directory(path).paths(
			# get recursively (bool).
			recursive=recursive,
			# get files only (bool).
			files_only=files_only,
			# get firs only (bool). 
			dirs_only=dirs_only,
			# also get empty dirs (bool).
			empty_dirs=empty_dirs,
			# the banned full paths (list).
			banned=banned,
			# the banned names (list).
			banned_names=banned_names,
			# the banend base names (list).
			banned_basenames=banned_basenames,
			# the allowed extensions (list).
			extensions=extensions, )
		if full_path:
			return paths
		else:
			_paths_ = []
			for i in paths: _paths_.append(self.subpath(i))
			return _paths_
		#

	# get the paths of a directory.
	def names(self,
		# the sub path (leave None to use the root path)
		path=None,
		# get recursively (bool).
		recursive=False, 
		# get files only (bool).
		files_only=False,
		# get firs only (bool). 
		dirs_only=False, 
		# also get empty dirs (bool).
		empty_dirs=True, 
		# remove the extension names (bool).
		remove_extensions=False,
		# the banned full paths (list).
		banned=[], 
		# the banned names (list).
		banned_names=[".DS_Store"], 
		# the banend base names (list).
		banned_basenames=["__pycache__"], 
		# the allowed extensions (list).
		extensions=["*"],
	):

		# checks.
		#if path == None: return _response_.error(self.__traceback__(function="names")+" Define parameter: [path].")
		if path == None: path = self.path
		else: path = self.join(path)
		if not Files.directory(path):
			raise dev0s.exceptions.InvalidUsage(f"Defined path [{path}] is not a directory.")

		# handler.
		return Directory(path).names(
			# get recursively (bool).
			recursive=recursive,
			# get files only (bool).
			files_only=files_only,
			# get firs only (bool). 
			dirs_only=dirs_only,
			# also get empty dirs (bool).
			empty_dirs=empty_dirs,
			# remove the extension names (bool).
			remove_extensions=remove_extensions,
			# the banned full paths (list).
			banned=banned,
			# the banned names (list).
			banned_names=banned_names,
			# the banend base names (list).
			banned_basenames=banned_basenames,
			# the allowed extensions (list).
			extensions=extensions,
		)
		#

	# representation.
	def __str__(self):
		return str(self.dir.fp.path)
	def __repr__(self):
		return str(self)

	# sys functions.
	def __serialize_format__(self, format):
		format = str(format).lower()
		if format in ["bytes", "Bytes", bytes, Bytes]: format = "bytes"
		elif format in ["bool", "Boolean", bool, Boolean]: format = "bool"
		elif format in ["string", "str", "String", str, String]: format = "str"
		elif format in ["int", "integer", int]: format = "int"
		elif format in ["float", float, Integer]: format = "float"
		elif format in ["list", "array", "Array", list, Array]: format = "list"
		elif format in ["dict", "dictionary", "Dictionary", dict, Dictionary]: format = "dict"
		if format not in ["bytes", "bool", "str", "int", "float", "dict", "list"]: raise Exceptions.InvalidUsage(f"{self.__traceback__()}: Format [{format}] is not a valid option, options: [bool, str, int, float, dict, list].")
		return format

	#

# the webserver database object class.
# keeps all info in python memory only.
class WebServer(Thread):
	def __init__(self,
		id="webserver",
		host="127.0.0.1",
		port=52379,
		sleeptime=3,
		log_level=0,
		# do not use.
		serialized={},
	):

		# docs.
		DOCS = {
			"module":"dev0s.database.WebServer", 
			"initialized":False,
			"description":[], 
			"chapter": "Database", }

		# defaults.
		Thread.__init__(self)

		# check home dir.
		for dir in [
			f"{defaults.vars.home}/.{ALIAS}",
			f"{defaults.vars.home}/.{ALIAS}/.cache",
		]:
			if not Files.exists(dir):
				Files.create(dir, directory=True, owner=defaults.vars.user, permission=700)

		# by serialization.
		if serialized != {}:
			self.assign(serialized, keys={
				"id":"webserver", 
				"host":"127.0.0.1", 
				"port":52379, 
				"sleeptime":3, 
				"log_level":0,
			})
		
		# by args.
		else:
			self.id = id
			self.host = host
			self.port = port
			self.sleeptime = sleeptime
			self.log_level = log_level

		# check parameters.
		_response_.parameters.check(
		    # the parameters (dict) (#1).
		    parameters={
		    	"id:str,String":self.id,
		    	"host:str,String":self.host,
		    	"port:str,String,int,float,Integer":self.port,
		    	"sleeptime:str,String,int,float,Integer":self.sleeptime,
		    	"log_level:str,String,int,float,Integer":self.log_level,
		    },
		    # the traceback id.
		    traceback=self.__traceback__(),).crash(error_only=True)

		# attribibutes.
		self.cache = {}
		self.system_cache = Database(path=Files.join(defaults.vars.home, f"/.{ALIAS}/.cache/{self.id}/")) # only used for tokens, rest is stored in python memory only.

		# checks.
		self.log_level = int(self.log_level)
		self.port = int(self.port)
		self.sleeptime = int(self.sleeptime)
		self.id = self.id.replace(" ","-")
		self.tag = self.id.replace(" ","_")

		#
	
	# cache functions.
	def set(self, group=None, id=None, data=None, timeout=3):
		encoded = _requests_.encode({
			"group":group.replace(" ","_"),
			"id":id.replace(" ","_"),
			"data":data,
			"token":self.token,
			"cache":self.system_cache.path,
			"cache_id":self.id,
		})
		try:
			response = __requests__.get(f'http://{self.host}:{self.port}/set?{encoded}', timeout=timeout)
		except Exception as e:
			return _response_.error(f"Failed to connect with {self.host}:{self.port}, error: {e}")
		try:
			response = self.__serialize__(response.json())
		except:
			return _response_.error(f"Failed to serialize {response}: {response.text}")
		return _response_.response(response)
	def get(self, group=None, id=None, timeout=3):
		encoded = _requests_.encode({
			"group":group.replace(" ","_"),
			"id":id.replace(" ","_"),
			"token":self.token,
			"cache":self.system_cache.path,
			"cache_id":self.id,
		})
		try:
			response = __requests__.get(f'http://{self.host}:{self.port}/get?{encoded}', timeout=timeout)
		except Exception as e:
			return _response_.error(f"Failed to connect with {self.host}:{self.port}, error: {e}")
		try:
			response = self.__serialize__(response.json())
		except:
			return _response_.error(f"Failed to serialize {response}: {response.text}")
		return _response_.response(response,)
		#

	# flask app.
	def app(self):
		app = flask.Flask(__name__)
		cli = sys.modules['flask.cli']
		cli.show_server_banner = lambda *x: None
		@app.route('/get')
		def get():
			token = flask.request.args.get('token')
			if token != Database(path=flask.request.args.get('cache')).load(Files.join(flask.request.args.get('cache_id'), "token")):
				return _response_.error(f"Provided an invalid token {token}.").json()
			group = flask.request.args.get('group')
			id = flask.request.args.get('id')
			if id in ["none", "null", "None"]: id = None
			try:
				if id == None:
					tag = f"{group}"
					value = self.cache[group]
				else:
					tag = f"{group}:{id}"
					value = self.cache[group][id]
			except KeyError:
				return _response_.error(f"There is no data cached for {tag}.").json()
			return _response_.success(f"Successfully retrieved {tag}.", {
				"group":group,
				"id":id,
				"data":value,
			}).json()
		@app.route('/set')
		def set():
			token = flask.request.args.get('token')
			if token != Database(path=flask.request.args.get('cache')).load(Files.join(flask.request.args.get('cache_id'), "token")):
				return _response_.error(f"Provided an invalid token {token}.").json()
			group = flask.request.args.get('group')
			id = flask.request.args.get('id')
			if id in ["none", "null", "None"]: id = None
			value = flask.request.args.get('data')
			if id == None:
				tag = f"{group}"
				self.cache[group] = value
			else:
				tag = f"{group}:{id}"
				try: self.cache[group]
				except KeyError: self.cache[group] = {}
				self.cache[group][id] = value
			return _response_.success(f"Successfully cached {tag}.").json()
		@app.route('/active')
		def active():
			token = flask.request.args.get('token')
			if token != Database(path=flask.request.args.get('cache')).load(Files.join(flask.request.args.get('cache_id'), "token")):
				return _response_.error(f"Provided an invalid token {token}.").json()
			return _response_.success(f"Active.").json()
		#def run__(self, app, host, port):
		#	app.run(host=host, port=port)
		#self.process = multiprocessing.Process(target=app.run, args=(self, app, self.host,self.port,))
		#self.process.start()
		app.run(host=self.host, port=self.port)
		#

	# control functions.
	def run(self):
		self.system_cache.save(path=f"{self.id}/daemon", data="*running*", format="str")
		self.system_cache.save(path=f"{self.id}/token", data=String().generate(length=64, digits=True, capitalize=True), format="str")
		self.app()
		self.system_cache.save(path=f"{self.id}/daemon", data="*stopped*", format="str")
	def fork(self, timeout=15, sleeptime=1):
		if self.running:
			return _response_.success(f"The {self.id} is already running.")
		if self.log_level <= 0:
			print(f"Starting the {self.id}.")
		serialized = self.dict(keys={
			"id":"webserver", 
			"host":"127.0.0.1", 
			"port":52379, 
			"sleeptime":3, 
			"log_level":0,
		})
		for i in ["__cache__", "cache", "system_cache" ,"_stder", "_traceback_", "_name", "_daemonic", "_ident", "_native_id", "_tstate_lock", "_started", "_stderr", "_initialized", "_invoke_excepthook", "__status__", "__running__", "__response__", "_is_stopped", "_args", "_kwargs", "_target", "_raw_traceback_",]:
			try: del serialized[i]
			except: a=1
		serialized = f"{serialized}"
		command = [str(defaults.vars.executable), f"{SOURCE_PATH}classes/database/fork.py", "--serialized", f"'{serialized}'", "--dev0s-webserver-tag", self.tag]
		if self.log_level < 0:
			command += [ "2>", "/dev/null"]
		p = subprocess.Popen(command)
		success = False
		for i in range(int(timeout/sleeptime)):
			if self.running:
				success = True
				break
			time.sleep(sleeptime)
		if success:
			return _response_.success(f"Successfully started the {self.id}.")
		else:
			return _response_.error(f"Failed to start the {self.id}.")
	def stop(self):
		if not self.running: 
			return _response_.success(f"{self.__traceback__(function='stop')}: The {self.id} is not running.")
		processes = code.processes(includes=f"--dev0s-webserver-tag {self.tag}")
		if not processes.success: return response
		if len(processes.processes) <= 1:
			return _response_.error(f"Unable to find the pid of the {self.id}.")
		for pid, info in processes.processes.items():
			if info["process"] not in ["grep"]:
				response = code.kill(pid=pid)
				if not response.success: return response
		return _response_.error(f"Successfully stopped the {self.id}.")
		#

	# threading functions.
	def start_thread(self, thread, group="daemons", id=None):
		response = self.set(group=group, id=id, data=thread)
		if not response.success: return response
		response = thread.start()
		if response != None:
			try: success = bool(response["success"])
			except: success = True
			if not success: return response
		try:
			id = thread.id
		except:
			id = str(thread)
		return _response_.success(f"Successfully started [{id}].")
	def get_thread(self, group="daemos", id=None):
		response = self.get(group=group, id=id)
		if not response.success: 
			if "There is no data cached for" in response.error:
				return _response_.error(f"There is no thread cached for (group: {group}), (id: {id}).")
			else: return response
		thread = response.data
		return _response_.success(f"Successfully retrieved thread [{thread}].", {
			"thread":thread,
		})
		#

	# properties.
	@property
	def token(self):
		if random.randrange(1, 100) <= 5: 
			response = self.system_cache.save(path=f"{self.id}/token", data=String().generate(length=64, digits=True, capitalize=True), format="str")
			if not response.success: response.crash()
		response = self.system_cache.load(path=f"{self.id}/token", format="str")
		if response.success and "None" not in str(response.data):
			return response.data
		else:
			response = self.system_cache.save(path=f"{self.id}/token", data=String().generate(length=64, digits=True, capitalize=True), format="str")
			if not response.success: response.crash()
		response = self.system_cache.load(path=f"{self.id}/token", format="str")
		if response.success and "None" not in str(response.data):
			return response.data
		else: 
			if response.error == None:
				response.error = f"Failed to create a token for webserver [{self.id}] ({self.system_cache.join(self.id+'/token')})."
			response.crash()
	@property
	def running(self):
		return self.__running__()
	def __running__(self, timeout=3):
		encoded = _requests_.encode({
			"token":self.token,
			"cache":self.system_cache.path,
			"cache_id":self.id,
		})
		try:
			__requests__.get(f'http://{self.host}:{self.port}/active?{encoded}', timeout=timeout)
			return True
		except __requests__.exceptions.ConnectionError:
			return False
		#

	# system functions.
	def __serialize__(self, dictionary, safe=False):
		if isinstance(dictionary, (Dictionary, dict)):
			new = {}
			for key, value in dictionary.items():
				if value in ["False", "false"]: new[key] = False
				elif value in ["True", "true"]: new[key] = True
				elif value in ["None", "none", "null", "nan"]: new[key] = None
				elif isinstance(value, (dict, Dictionary, list, Array)):
					new[key] = self.__serialize__(value, safe=safe)
				elif isinstance(value, object):
					if not safe:
						new[key] = value
				else:
					try: 
						int(value)
						new[key] = int(value)
					except: 
						new[key] = value
			return new
		if isinstance(dictionary, (Dictionary, dict)):
			new = []
			for value in dictionary:
				if value in ["False", "false"]: new.append(False)
				elif value in ["True", "true"]: new.append(True)
				elif value in ["None", "none", "null", "nan"]: new.append(None)
				elif isinstance(value, (dict, Dictionary)):
					new.append(self.__serialize__(value, safe=safe))
				elif isinstance(value, object):
					if not safe:
						new.append(value)
				else:
					try: 
						int(value)
						new.append(int(value))
					except: 
						new.append(value)
			return new
		else: raise ValueError(f"Parameter [dictionary] requires to be a [dict, Dictionary, list, Array], not [{dictionary.__class__.__name__}].")

	#

#
			



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# imports.
from dev0s.classes.defaults.files import *
from dev0s.classes.response import response as _response_

# the installation object class.
class Installation(object):
	def __init__(self):	

		if OS in ["macos"]: 
			self.bashrc = File(path=f"/Users/{USER}/.zshrc", load=True)
			self.link = f"source {SOURCE_PATH}/lib/bash/import\nsource {SOURCE_PATH}/lib/bash/env"
		else: 
			self.bashrc = File(path=f"/home/{USER}/.bashrc", load=True)
			self.link = f". {SOURCE_PATH}/lib/bash/import\n. {SOURCE_PATH}/lib/bash/env"
		#
	def link(self):
		self.bashrc.load()
		if self.link not in self.bashrc.data:
			self.bashrc.data += f"\n\n# DevOS:\n{self.link}\n"
			self.bashrc.save()
		return _response_.success(f"Successfully linked {ALIAS}.")
	def unlink(self):
		self.bashrc.load()
		if self.link in file.data:
			data = ""
			for line in self.bashrc.data.split("\n"):
				if "# DevOS:" in line or "# dev0s:" in line or "# dev0s." in line or self.link in line:
					a=1
				else:
					data += line+"\n"
			self.bashrc.save(data)
		return _response_.success(f"Successfully unlinked {ALIAS}.")
	def install(self):
		os.system("rm -fr /tmp/dev0s && git clone -q https://github.com/vandenberghinc/dev0s.git /tmp/dev0s && bash /tmp/dev0s/requirements/installer")
		return _response_.success(f"Successfully installed {ALIAS}.")
	def uninstall(self):
		print(f"{color.orange}Root permissions{color.end} required to uninstall DevOS.")
		os.system(f"sudo rm -fr /usr/local/lib/{ALIAS}")
		return _response_.success(f"Successfully uninstalled {ALIAS}.")
	# system functions.
	def __installed__(self):
		return Files.exists(f"/usr/local/lib/{ALIAS}")





# imports.
from dev0s.classes.defaults.files import Date
from dev0s.classes.response import response as _response_
import json, requests

# the telgram object.
class Telegram(object):
	def __init__(self,
		# your telegram bot token.
		token=None,
	):

		# check params.
		response = _response_.parameters.check({
			"token:str":token,
		})
		if not response.success: response.crash()

		# attributes.
		self.token = token

	# read telegram messages.
	def read(self, 
		# optionally specify a chat id (str).
		chat_id=None,
	):

		# request.
		response = self.__request__(f"https://api.telegram.org/bot{self.token}/getUpdates")
		#response = self.__request__(f"https://api.telegram.org/bot{self.token}/getMessages")
		if not response.success: return response
		response = response["response"]

		# read all messages.
		messages = {}
		for response_data in response['result']:

			# parse.
			from_ = response_data["message"]["from"]
			chat = response_data["message"]["chat"]
			date = Date().from_seconds(response_data["message"]["date"])
			message = response_data["message"]["text"]

			# append.
			try: messages[chat['id']]
			except KeyError: messages[chat['id']] = {}
			messages[chat['id']][str(date)] = {
				"message":message,
				"date":str(date),
				"from":from_,
				"chat":chat,
			}
		
		# handler.
		if chat_id == None:
			return _response_.success("Successfully retrieved the messages.", {
				"messages":messages,
			})
		else:
			try:
				messages = messages[chat_id]
			except KeyError:
				messages = {}
			return _response_.success(f"Successfully retrieved the messages from chat {chat_id}.", {
				"messages":messages,
			})

		#

	# send a telegram message.
	def send(self, 
		# the message to send (str).
		message=None, 
		# the chat id where the message will be sent to.
		chat_id=None,
		# the message title.
		title=None,
	):

		# check params.
		response = _response_.parameters.check({
			"message:str":message,
			"chat_id:str":chat_id,
		})
		if not response.success: return response

		# clean msg.
		message = message.replace('[', '(').replace(']', ')')

		# make request.
		if title not in [None, False, "", "null", "NaN"]:
			message = f"*{title}*\n" + message
		response = self.__request__(f"https://api.telegram.org/bot{self.token}/sendMessage?chat_id={chat_id}&parse_mode=Markdown&text={message}")
		if not response.success: return response

		# handler.
		return _response_.success(f"Successfull send the message to {chat_id}.")

		#

	# make an request.
	def __request__(self, url: str, serialize=True):
		# make request.
		response = requests.get(url)

		# check status code.
		if response.status_code != 200:
			return _response_.error(f"Invalid request ({url}) [{response.status_code}]: {response.text}.")

		# serialize.
		if serialize:
			try: response = response.json()
			except: 
				return _response_.error(f"Failed to serialize response: {response.text}.")

		# handler.
		return _response_.success("Successfull request", {
			"response":response,
		})

		#

	#




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# imports (over here to prevent circular).
from dev0s.classes.system import disks, env, service, system, browser, metrics, cleanup

# the system module class.
class System():

	# disks.
	disks = disks.disks

	# env.
	env = env.env

	# service.
	Service = service.Service

	# system.
	check_user = system.check_user
	check_group = system.check_group
	User = system.User
	Group = system.Group

	# browser.
	Browser = browser.Browser

	# metrics.
	metrics = metrics.Metrics()

	# cleanup.
	cleanup = cleanup.CleanUp()
